@@ -481,7 +481,7 @@ static void prepare_pcap(void) {
     if (!use_iface) {
 
       /* See the earlier note on libpcap SEGV - same problem here.
-         Also, this retusns something stupid on Windows, but hey... */
+         Also, this returns something stupid on Windows, but hey... */
      
       if (!access("/sys/class/net", R_OK | X_OK) || errno == ENOENT)
         use_iface = (u8*)pcap_lookupdev(pcap_err);
@@ -545,6 +545,8 @@ static void prepare_bpf(void) {
 
   vlan_support = (pcap_datalink(pt) == DLT_EN10MB);
 
+retry_no_vlan:
+
   if (!orig_rule) {
 
     if (vlan_support) {
@@ -575,6 +577,15 @@ static void prepare_bpf(void) {
   DEBUG("[#] Computed rule: %s\n", final_rule);
 
   if (pcap_compile(pt, &flt, (char*)final_rule, 1, 0)) {
+
+    if (vlan_support) {
+
+      if (orig_rule) ck_free(final_rule);
+      vlan_support = 0;
+      goto retry_no_vlan;
+
+    }
+
     pcap_perror(pt, "[-] pcap_compile");
 
     if (!orig_rule)
@@ -812,6 +823,30 @@ poll_again:
 
     for (cur = 0; cur < pfd_count; cur++) {
 
+      if (pfds[cur].revents & (POLLERR | POLLHUP)) switch (cur) {
+
+        case 0:
+
+          FATAL("Packet capture interface is down.");
+
+        case 1:
+
+          FATAL("API socket is down.");
+
+        default:
+
+          /* Shut down API connection and free its state. */
+
+          DEBUG("[#] API connection on fd %d closed.\n", pfds[cur].fd);
+
+          close(pfds[cur].fd);
+          ctable[cur]->fd = -1;
+ 
+          pfd_count = regen_pfds(pfds, ctable);
+          goto poll_again;
+
+      }
+
       if (pfds[cur].revents & POLLOUT) switch (cur) {
 
         case 0: case 1:
@@ -917,29 +952,6 @@ poll_again:
 
       }
 
-      if (pfds[cur].revents & (POLLERR | POLLHUP)) switch (cur) {
-
-        case 0:
-
-          FATAL("Packet capture interface is down.");
-
-        case 1:
-
-          FATAL("API socket is down.");
-
-        default:
-
-          /* Shut down API connection and free its state. */
-
-          DEBUG("[#] API connection on fd %d closed.\n", pfds[cur].fd);
-
-          close(pfds[cur].fd);
-          ctable[cur]->fd = -1;
- 
-          pfd_count = regen_pfds(pfds, ctable);
-          goto poll_again;
-
-      }
 
       /* Processed all reported updates already? If so, bail out early. */
 
@@ -949,6 +961,9 @@ poll_again:
 
   }
 
+  ck_free(ctable);
+  ck_free(pfds);
+
 #else
 
   if (!daemon_mode) 
@@ -974,9 +989,6 @@ poll_again:
 
   WARN("User-initiated shutdown.");
 
-  ck_free(ctable);
-  ck_free(pfds);
-
 }
 
 
@@ -1088,7 +1100,7 @@ int main(int argc, char** argv) {
     case 'p':
     
       if (set_promisc)
-        FATAL("Even more promiscuous? People will call me slutty!");
+        FATAL("Even more promiscuous? People will start talking!");
 
       set_promisc = 1;
       break;
